@@ -9,10 +9,13 @@ from nonebot_plugin_imageutils import BuildImage, text2image
 
 from .color_table import color_table
 from .depends import Arg, Img, NoArg
-from .utils import save_gif, make_jpg_or_gif, Maker, get_avg_duration
+from .utils import save_gif, make_jpg_or_gif, Maker, get_avg_duration, split_gif
 
 colors = "|".join(colormap.keys())
-color_pattern = rf"#[a-fA-F0-9]{{6}}|{colors}"
+color_pattern_str = rf"#[a-fA-F0-9]{{6}}|{colors}"
+
+num_256 = r"(25[0-5]|2[0-4][0-9]|[0-1]?[0-9]?[0-9])"
+color_pattern_num = rf"(?:rgb)?\(?\s*{num_256}[\s,]+{num_256}[\s,]+{num_256}\s*\)?;?"
 
 
 def flip_horizontal(img: BuildImage = Img(), arg=NoArg()):
@@ -24,7 +27,7 @@ def flip_vertical(img: BuildImage = Img(), arg=NoArg()):
 
 
 def grey(img: BuildImage = Img(), arg=NoArg()):
-    return make_jpg_or_gif(img, lambda img: img.convert("L"))
+    return make_jpg_or_gif(img, lambda img: img.convert("L"), keep_transparency=False)
 
 
 def rotate(img: BuildImage = Img(), arg: str = Arg()):
@@ -80,9 +83,12 @@ def crop(img: BuildImage = Img(), arg: str = Arg()):
 
 
 def invert(img: BuildImage = Img(), arg=NoArg()):
-    return make_jpg_or_gif(
-        img, lambda img: BuildImage(ImageOps.invert(img.convert("RGB").image))
-    )
+    def make(img: BuildImage) -> BuildImage:
+        result = BuildImage.new("RGB", img.size, "white")
+        result.paste(img, alpha=True)
+        return BuildImage(ImageOps.invert(result.image))
+
+    return make_jpg_or_gif(img, make, keep_transparency=False)
 
 
 def contour(img: BuildImage = Img(), arg=NoArg()):
@@ -120,61 +126,68 @@ def pixelate(img: BuildImage = Img(), arg: str = Arg()):
 
 
 def color_mask(img: BuildImage = Img(), arg: str = Arg()):
-    if re.fullmatch(color_pattern, arg):
+    if re.fullmatch(color_pattern_str, arg):
         color = arg
+    elif match := re.fullmatch(color_pattern_num, arg):
+        color = tuple(map(int, match.groups()))
     elif arg in color_table:
         color = color_table[arg]
     else:
-        return "请使用正确的颜色格式，如：#66ccff、red、红色"
+        return "请使用正确的颜色格式，如：#66ccff、red、红色、102 204 255"
     return make_jpg_or_gif(img, lambda img: img.color_mask(color))
 
 
 def color_image(arg: str = Arg()):
-    if re.fullmatch(color_pattern, arg):
+    if re.fullmatch(color_pattern_str, arg):
         color = arg
+    elif match := re.fullmatch(color_pattern_num, arg):
+        color = tuple(map(int, match.groups()))
     elif arg in color_table:
         color = color_table[arg]
     else:
-        return "请使用正确的颜色格式，如：#66ccff、red、红色"
+        return "请使用正确的颜色格式，如：#66ccff、red、红色、102 204 255"
     return BuildImage.new("RGB", (500, 500), color).save_jpg()
 
 
 def gif_reverse(img: BuildImage = Img(), arg=NoArg()):
     image = img.image
     if getattr(image, "is_animated", False):
+        frames = split_gif(image)
         duration = get_avg_duration(image) / 1000
-        image.seek(0)
-        transparency = image.info.get("transparency", 0)
-        frames: List[IMG] = []
-        for i in reversed(range(image.n_frames)):
-            image.seek(i)
-            frames.append(image.convert("RGBA"))
-        frames[0].info["transparency"] = transparency
-        return save_gif(frames, duration)
+        return save_gif(frames[::-1], duration)
 
 
 def gif_obverse_reverse(img: BuildImage = Img(), arg=NoArg()):
     image = img.image
     if getattr(image, "is_animated", False):
+        frames = split_gif(image)
         duration = get_avg_duration(image) / 1000
-        frames: List[IMG] = []
-        for i in range(image.n_frames):
-            image.seek(i)
-            frames.append(image.convert("RGBA"))
-        frames = frames + frames[::-1]
-        image.seek(0)
-        frames[0].info["transparency"] = image.info.get("transparency", 0)
+        frames = frames + frames[-2::-1]
         return save_gif(frames, duration)
 
 
+def gif_change_fps(img: BuildImage = Img(), arg: str = Arg()):
+    image = img.image
+    if not getattr(image, "is_animated", False):
+        return
+    match1 = re.fullmatch(r"([\d\.]{1,4})(?:x|X|倍速?)", arg)
+    match2 = re.fullmatch(r"(\d{1,3})%", arg)
+    duration = get_avg_duration(image) / 1000
+    if match1:
+        duration /= float(match1.group(1))
+    elif match2:
+        duration /= int(match2.group(1)) / 100
+    else:
+        return "请使用正确的倍率格式，如：0.5x、50%"
+    frames = split_gif(image)
+    return save_gif(frames, duration)
+
+
 def gif_split(img: BuildImage = Img(), arg=NoArg()):
     image = img.image
     if getattr(image, "is_animated", False):
-        output: List[BytesIO] = []
-        for i in range(image.n_frames):
-            image.seek(i)
-            output.append(BuildImage(image.convert("RGB")).save_png())
-        return output
+        frames = split_gif(image)
+        return [BuildImage(frame).save_png() for frame in frames]
 
 
 def four_grid(img: BuildImage = Img(), arg=NoArg()):
